<?php

/**
 * @file
 * Contains the form options views style plugin.
 */

/**
 * Style plugin to render each row as an option for a form element.
 *
 * @ingroup views_style_plugins
 */
class vspfo_form_options extends views_plugin_style {

  /**
   * {@inheritdoc}
   */
  public function option_definition() {
    $options = parent::option_definition();

    $options['value_field'] = array('default' => '');
    $options['element_type'] = array('default' => 'select');

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $handlers = $this->display->handler->get_handlers('field');
    $fields = array();

    if (empty($handlers)) {
      $form['error_markup'] = array(
        '#markup' => t('You need at least one field before you can configure your form options settings.'),
        '#prefix' => '<div class="error messages">',
        '#suffix' => '</div>',
      );
      return;
    }

    foreach ($handlers as $id => $handler) {
      $fields[$id] = $handler->ui_name();
    }

    $form['markup'] = array(
      '#markup' => t('To properly configure form options, you must select one field that will represent the option value to utilize. You can then set that field to exclude from output. All other displayed fields will be part of the option. Please note that all HTML will be stripped from the output in the select box.'),
      '#prefix' => '<div class="form-item description">',
      '#suffix' => '</div>',
    );

    $form['value_field'] = array(
      '#type' => 'select',
      '#title' => t('Option value'),
      '#options' => $fields,
      '#default_value' => $this->options['value_field'],
    );

    $form['element_type'] = array(
      '#type' => 'radios',
      '#title' => t('Element type'),
      '#options' => array(
        'checkboxes' => t('Checkboxes'),
        'radios' => t('Radios'),
        'select' => t('Select'),
      ),
      '#default_value' => $this->options['element_type'],
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $sets = $this->render_grouping($this->view->result, $this->options['grouping'], TRUE);
    $options = $this->process_grouping_sets($sets);

    $form_id = "vspfo_form_{$this->view->name}_{$this->display->id}";
    $settings = array(
      'options' => $options,
      'element_type' => $this->options['element_type'],
      'ajax' => $this->view->use_ajax || !empty($this->view->live_preview),
    );
    return drupal_get_form($form_id, $settings);
  }

  /**
   * Processes the grouping sets.
   *
   * @param array $sets
   *   Array containing the grouping sets.
   *
   * @return array
   *   Grouping sets as nested form options array.
   */
  public function process_grouping_sets(array $sets) {
    $options = array();

    foreach ($sets as $set) {
      $row = reset($set['rows']);
      // Process as a grouping set.
      if (is_array($row) && isset($row['group'])) {
        $options[$set['group']] = $this->process_grouping_sets($set['rows']);
      }
      // Process as a record set.
      else {
        foreach ($set['rows'] as $index => $row) {
          $this->view->row_index = $index;

          $option = $this->row_plugin->render($row);
          if ($this->options['element_type'] == 'select') {
            $option = trim(strip_tags(decode_entities($option)));
          }

          if (!empty($this->options['value_field'])) {
            $value = $this->get_field($this->view->row_index, $this->options['value_field']);
          }
          else {
            // Initially value field option may be missing.
            $value = $option;
          }

          if ($set['group']) {
            $options[$set['group']][$value] = $option;
          }
          else {
            $options[$value] = $option;
          }
        }
      }
    }
    unset($this->view->row_index);

    return $options;
  }

}
